@@ -282,7 +282,7 @@ class Launch(FWSerializable, object):
         Updates the update_at field of the state history of a Launch. Used to
          ping that a Launch is still alive.
         """
-        udpate_time = update_time if update_time else datetime.datetime.utcnow()
+        update_time = update_time if update_time else datetime.datetime.utcnow()
         self.state_history[-1]['updated_on'] = update_time
 
     def set_reservation_id(self, reservation_id):
