@@ -4,15 +4,17 @@
 The LaunchPad manages the FireWorks database.
 """
 import datetime
+import json
 import os
 import time
+import traceback
 
 from pymongo.mongo_client import MongoClient
 from pymongo import DESCENDING
 
 from fireworks.core.fw_config import FWConfig
 from fireworks.utilities.fw_serializers import FWSerializable
-from fireworks.core.firework import FireWork, Launch, Workflow
+from fireworks.core.firework import FireWork, Launch, Workflow, FWAction
 from fireworks.utilities.fw_utilities import get_fw_logger
 
 
@@ -27,7 +29,7 @@ __date__ = 'Jan 30, 2013'
 # TODO: lots of duplication reduction and cleanup possible
 
 # TODO: can actions like complete_launch() be done as a transaction? e.g. refresh_wf() might have error...I guess at
-# least set the state to FAILED or ERROR and add traceback...
+# least set the state to FIZZLED or ERROR and add traceback...
 
 # Note: Always use find_and_modify() for *all* database updates. Otherwise you will run into synchronization /
 # delayed write errors from Mongo. Even find_and_modify() does not guarantee zero errors, but it reduces incidents
@@ -75,6 +77,7 @@ class LaunchPad(FWSerializable):
 
         self.fireworks = self.db.fireworks
         self.launches = self.db.launches
+        self.offline_runs = self.db.offline_runs
         self.fw_id_assigner = self.db.fw_id_assigner
         self.workflows = self.db.workflows
 
@@ -121,6 +124,7 @@ class LaunchPad(FWSerializable):
             self.fireworks.remove()
             self.launches.remove()
             self.workflows.remove()
+            self.offline_runs.remove()
             self._restart_ids(1, 1)
             self.tuneup()
             self.m_logger.info('LaunchPad was RESET.')
@@ -133,10 +137,10 @@ class LaunchPad(FWSerializable):
         while True:
             self.m_logger.info('Performing maintenance on Launchpad...')
 
-            self.m_logger.debug('Tracking down FAILED jobs...')
-            fl = self.detect_failed(fix=True)
+            self.m_logger.debug('Tracking down FIZZLED jobs...')
+            fl = self.detect_fizzled(fix=True)
             if fl:
-                self.m_logger.info('Detected {} FAILED launches: {}'.format(len(fl), fl))
+                self.m_logger.info('Detected {} FIZZLED launches: {}'.format(len(fl), fl))
 
             self.m_logger.debug('Tracking down stuck RESERVED jobs...')
             ur = self.detect_unreserved(fix=True)
@@ -310,7 +314,7 @@ class LaunchPad(FWSerializable):
 
 
     def defuse_fw(self, fw_id):
-        allowed_states = ['DEFUSED', 'WAITING', 'READY', 'FAILED']
+        allowed_states = ['DEFUSED', 'WAITING', 'READY', 'FIZZLED']
         f = self.fireworks.find_and_modify({'fw_id': fw_id, 'state': {'$in': allowed_states}},
                                               {'$set': {'state': 'DEFUSED'}})
 
@@ -445,12 +449,12 @@ class LaunchPad(FWSerializable):
                 self.unreserve(lid)
         return bad_launch_ids
 
-    def mark_failed(self, launch_id):
+    def mark_fizzled(self, launch_id):
         # TODO: this seems a lot like the code in complete_launch...DRY
 
         # Do a confirmed write and make sure state_history is preserved
         m_launch = self.get_launch_by_id(launch_id)
-        m_launch.state = 'FAILED'
+        m_launch.state = 'FIZZLED'
         self._upsert_launch(m_launch)
 
         for fw_data in self.fireworks.find({'launches': launch_id}, {'fw_id': 1}):
@@ -458,7 +462,7 @@ class LaunchPad(FWSerializable):
             wf = self.get_wf_by_fw_id(fw_id)
             self._refresh_wf(wf, fw_id)
 
-    def detect_failed(self, expiration_secs=FWConfig().RUN_EXPIRATION_SECS, fix=False):
+    def detect_fizzled(self, expiration_secs=FWConfig().RUN_EXPIRATION_SECS, fix=False):
         bad_launch_ids = []
         now_time = datetime.datetime.utcnow()
         cutoff_timestr = (now_time - datetime.timedelta(seconds=expiration_secs)).isoformat()
@@ -469,7 +473,7 @@ class LaunchPad(FWSerializable):
             bad_launch_ids.append(ld['launch_id'])
         if fix:
             for lid in bad_launch_ids:
-                self.mark_failed(lid)
+                self.mark_fizzled(lid)
         return bad_launch_ids
 
     def _set_reservation_id(self, launch_id, reservation_id):
@@ -523,7 +527,7 @@ class LaunchPad(FWSerializable):
 
         # update any duplicated runs
         for fw in self.fireworks.find(
-                {'launches': l_id, 'state': {'$in': ['WAITING', 'READY', 'RESERVED', 'FAILED']}},
+                {'launches': l_id, 'state': {'$in': ['WAITING', 'READY', 'RESERVED', 'FIZZLED']}},
                 {'fw_id': 1}):
             fw_id = fw['fw_id']
             fw = self.get_fw_by_id(fw_id)
@@ -556,9 +560,11 @@ class LaunchPad(FWSerializable):
             fw_id = fw['fw_id']
             self._refresh_wf(self.get_wf_by_fw_id(fw_id), fw_id)
 
-    def ping_launch(self, launch_id):
+        return m_launch
+
+    def ping_launch(self, launch_id, ptime=None):
         m_launch = self.get_launch_by_id(launch_id)
-        m_launch.touch_history()
+        m_launch.touch_history(ptime)
         self.launches.find_and_modify({'launch_id': launch_id, 'state': 'RUNNING'},
             {'$set':{'state_history':m_launch.to_db_dict()['state_history']}})
 
@@ -647,10 +653,66 @@ class LaunchPad(FWSerializable):
         return stolen
 
     def _upsert_launch(self, m_launch):
-        # Do a confirmed write of Launch
         # TODO: this no longer needs to be its own function (much was removed)
         self.launches.find_and_modify({'launch_id': m_launch.launch_id}, m_launch.to_db_dict(), upsert=True)
 
+    def set_priority(self, fw_id, priority):
+        self.fireworks.find_and_modify({"fw_id": fw_id}, {'$set': {'spec._priority': priority}})
+
     def get_logdir(self):
         # AJ: This is needed for job packing due to Proxy objects not being fully featured...
-        return self.logdir
\ No newline at end of file
+        return self.logdir
+
+    def add_offline_run(self, launch_id, fw_id, name):
+        d = {'fw_id': fw_id}
+        d['launch_id'] = launch_id
+        d['name'] = name
+        d['created_on'] = datetime.datetime.utcnow().isoformat()
+        d['updated_on'] = datetime.datetime.utcnow().isoformat()
+        d['deprecated'] = False
+        d['completed'] = False
+        self.offline_runs.insert(d)
+
+    def recover_offline(self, launch_id, ignore_errors=False):
+        # get the launch directory
+        m_launch = self.get_launch_by_id(launch_id)
+        try:
+            self.m_logger.debug("RECOVERING fw_id: {}".format(m_launch.fw_id))
+            # look for ping file - update the FireWork if this is the case
+            ping_loc = os.path.join(m_launch.launch_dir, "FW_ping.json")
+            if os.path.exists(ping_loc):
+                with open(ping_loc) as f:
+                    ping_time = datetime.datetime.strptime(json.loads(f.read())['ping_time'], "%Y-%m-%dT%H:%M:%S.%f")
+                    self.ping_launch(launch_id, ping_time)
+
+            # look for action in FW_offline.json
+            offline_loc = os.path.join(m_launch.launch_dir, "FW_offline.json")
+            with open(offline_loc) as f:
+                offline_data = json.loads(f.read())
+                if 'started_on' in offline_data:
+                    m_launch.state = 'RUNNING'
+                    for s in m_launch.state_history:
+                        if s['state'] == 'RUNNING':
+                            s['created_on'] = datetime.datetime.strptime(offline_data['started_on'], "%Y-%m-%dT%H:%M:%S.%f")
+                    self._upsert_launch(m_launch)
+
+                if 'fwaction' in offline_data:
+                    fwaction = FWAction.from_dict(offline_data['fwaction'])
+                    state = offline_data['state']
+                    m_launch = self.complete_launch(launch_id, fwaction, state)
+                    for s in m_launch.state_history:
+                        if s['state'] == offline_data['state']:
+                            s['created_on'] = datetime.datetime.strptime(offline_data['completed_on'], "%Y-%m-%dT%H:%M:%S.%f")
+                    self._upsert_launch(m_launch)
+                    self.offline_runs.update({"launch_id": launch_id}, {"$set": {"completed":True}})
+
+            # update the updated_on
+            self.offline_runs.update({"launch_id": launch_id}, {"$set": {"updated_on": datetime.datetime.utcnow().isoformat()}})
+            return None
+        except:
+            if not ignore_errors:
+                traceback.print_exc()
+            return m_launch.fw_id
+
+    def forget_offline(self, fw_id):
+        self.offline_runs.update({"fw_id": fw_id}, {"$set": {"deprecated":True}})
\ No newline at end of file
